<?php

namespace Drupal\testsuite;

use Drupal\Core\Database\Connection;
use Drupal\Core\Session\AccountInterface;
use Drupal\testsuite\BaseService;

/**
 * Test Suite Services.
 */
class LoadResourceService extends BaseService {

    /**
     * @var array List of directories for the menu.
     */
    protected $_menuArray = array(
        'custom' => array(),
        'contrib' => array(),
        'core' => array()
    );

    /**
     * @var array List of custom tests.
     */
    protected $_customTestsArray = array();

    /**
     * @var array List of phpunit tests.
     */
    protected $_formReportArray = array();

    /**
     * @var array List of links.
     */
    protected $_listFilesArray = array();

    /**
     * @var \Drupal\Core\Database\Connection
     */
    protected $_connection;

    /**
     * @var \Drupal\Core\Session\AccountInterface
     */
    protected $_account;

    /**
     * @var string Absolute path to the resource.
     */
    protected $_path;

    /**
     * @param \Drupal\Core\Database\Connection $connection
     * @param \Drupal\Core\Session\AccountInterface $account
     */
    public function __construct(Connection $connection, AccountInterface $account) {
        $this->_connection = $connection;
        $this->_account = $account;
        $this->_path = dirname(dirname(dirname(dirname(__DIR__))));
    }

    /**
     * Gets the array of files to create classes with.
     *
     * @param string $caller
     *   The page type. Menu, form, report etc.
     * @param string $testType
     *   The type of test. Custom or phpunit tests.
     * @param string $area
     *   The area the module is located. Core, contrib or custom.
     * @param string $module
     *   The module name.
     * @param string $directory
     *   The test directory. Unit, Functional etc.
     *
     * @return array
     *   The array of files to create classes with.
     */
    public function getResource($caller, $testType, $area, $module, $directory) {
        switch($testType) {
            case 'custom':
                $this->_getCustomTests('custom', $this->_path . '/modules/custom');
                return $this->_customTestsArray;
            case 'phpunit':
                if($caller == 'menu') {
                    $this->_getDirectories('core', $this->_path . '/core/modules');
                    $this->_getDirectories('contrib', $this->_path . '/modules/contrib');
                    $this->_getDirectories('custom', $this->_path . '/modules/custom');
                    return $this->_menuArray;
                } else {
                    switch($area) {
                        case 'core'; $this->_getFormReportArray('core', $module, $directory, $this->_path . '/core/modules'); break;
                        case 'contrib'; $this->_getFormReportArray('contrib', $module, $directory, $this->_path . '/modules/contrib'); break;
                        case 'custom'; $this->_getFormReportArray('custom', $module, $directory, $this->_path . '/modules/custom'); break;
                    }
                    return $this->_formReportArray;
                }
        }
    }

    /**
     * Returns the phpunit statement with a file.
     *
     * @param string $area
     *   The area the module is located. Core, contrib or custom.
     * @param string $module
     *   The module name.
     * @param string $directory
     *   The test directory. Unit, Functional etc.
     * @param string $filePath
     *   The absolute path the tests.
     *
     * @return string
     *   The statement to run a phpunit test.
     */
    public function getFileStatement($area, $module, $directory, $filePath) {
        $parts = explode("/", $filePath);
        $file = end($parts);
        // dd($file);
        if($this->hasWebFolder()) {
            if($this->isWindows()) {
                switch($area) {
                    case 'core': return 'vendor\bin\phpunit --debug -c web\core web\\'.$area.'\modules\\'.$module.'\tests\src\\'.$directory.'\\'.$file;
                    case 'contrib': return 'vendor\bin\phpunit --debug -c web\core web\modules\contrib\\'.$module.'\tests\src\\'.$directory.'\\'.$file;
                    case 'custom': return 'vendor\bin\phpunit --debug -c web\core web\modules\custom\\'.$module.'\tests\src\\'.$directory.'\\'.$file;
                }
            } else {
                switch($area) {
                    case 'core': return 'vendor/bin/phpunit --debug -c web/core web/'.$area.'/modules/'.$module.'/tests/src/'.$directory.'/'.$file;
                    case 'contrib': return 'vendor/bin/phpunit --debug -c web/core web/modules/contrib/'.$module.'/tests/src/'.$directory.'/'.$file;
                    case 'custom': return 'vendor/bin/phpunit --debug -c web/core web/modules/custom/'.$module.'/tests/src/'.$directory.'/'.$file;
                }
            }
        } else {
            if($this->isWindows()) {
                switch($area) {
                    case 'core': return 'vendor\bin\phpunit --debug -c core '.$area.'\modules\\'.$module.'\tests\src\\'.$directory.'\\'.$file;
                    case 'contrib': return 'vendor\bin\phpunit --debug -c core modules\contrib\\'.$module.'\tests\src\\'.$directory.'\\'.$file;
                    case 'custom': return 'vendor\bin\phpunit --debug -c core modules\custom\\'.$module.'\tests\src\\'.$directory.'\\'.$file;
                }
            } else {
                switch($area) {
                    case 'core': return 'vendor/bin/phpunit --debug -c core '.$area.'/modules/'.$module.'/tests/src/'.$directory.'/'.$file;
                    case 'contrib': return 'vendor/bin/phpunit --debug -c core modules/contrib/'.$module.'/tests/src/'.$directory.'/'.$file;
                    case 'custom': return 'vendor/bin/phpunit --debug -c core modules/custom/'.$module.'/tests/src/'.$directory.'/'.$file;
                }
            }
        }
    }

    /**
     * Returns the link to a report by file.
     *
     * @param string $area
     *   The area the module is located. Core, contrib or custom.
     * @param string $module
     *   The module name.
     * @param string $directory
     *   The test directory. Unit, Functional etc.
     * @param string $statement
     *   The statement used in Phpunit to call tests from the CMD.
     *
     * @return string
     *   The link to a report.
     */
    private function getUrls($area, $module, $directory, $statement = null) {
        if($statement) {
            $parts = explode("/", $statement);
            $link = \Drupal::request()->getSchemeAndHttpHost() ."/admin/testsuite/phpunit/file/report/". $area ."/". $module ."/". $directory ."/". end($parts);
        } else {
            $link = \Drupal::request()->getSchemeAndHttpHost() ."/admin/testsuite/phpunit/file/report/". $area ."/". $module ."/". $directory;
        }

        return $link;
    }

    /**
     * Returns the files listed under each directory in the phpunit form that create exterior links to report.
     *
     * @param string $area
     *   The area the module is located. Core, contrib or custom.
     * @param string $module
     *   The module name.
     * @param string $directory
     *   The test directory. Unit, Functional etc.
     *
     * @return array
     *   The list of links to reports by file.
     */
    public function getFiles($area, $module, $directory) {
        $path = "";
        switch($area) {
            case 'core'; $path = $this->_path . '/core/modules'; break;
            case 'contrib'; $path = $this->_path . '/modules/contrib'; break;
            case 'custom'; $path = $this->_path . '/modules/custom'; break;
        }
        $this->_getFilesInDirectories($area, $module, $directory, $path);
        return $this->_listFilesArray;
    }

    /**
     * Returns the phpunit statement directory.
     *
     * @param string $area
     *   The area the module is located. Core, contrib or custom.
     * @param string $module
     *   The module name.
     * @param string $directory
     *   The test directory. Unit, Functional etc.
     *
     * @return string
     *   The statement to run a phpunit test.
     */
    public function getStatement($area, $module, $directory) {
        if($this->hasWebFolder()) {
            if($this->isWindows()) {
                switch($area) {
                    case 'core': return 'vendor\bin\phpunit --debug -c web\core web\\'.$area.'\modules\\'.$module.'\tests\src\\'.$directory;
                    case 'contrib': return 'vendor\bin\phpunit --debug -c web\core web\modules\contrib\\'.$module.'\tests\src\\'.$directory;
                    case 'custom': return 'vendor\bin\phpunit --debug -c web\core web\modules\custom\\'.$module.'\tests\src\\'.$directory;
                }
            } else {
                switch($area) {
                    case 'core': return 'vendor/bin/phpunit --debug -c web/core web/'.$area.'/modules/'.$module.'/tests/src/'.$directory;
                    case 'contrib': return 'vendor/bin/phpunit --debug -c web/core web/modules/contrib/'.$module.'/tests/src/'.$directory;
                    case 'custom': return 'vendor/bin/phpunit --debug -c web/core web/modules/custom/'.$module.'/tests/src/'.$directory;
                }
            }
        } else {
            if($this->isWindows()) {
                switch($area) {
                    case 'core': return 'vendor\bin\phpunit --debug -c core '.$area.'\modules\\'.$module.'\tests\src\\'.$directory;
                    case 'contrib': return 'vendor\bin\phpunit --debug -c core modules\contrib\\'.$module.'\tests\src\\'.$directory;
                    case 'custom': return 'vendor\bin\phpunit --debug -c core modules\custom\\'.$module.'\tests\src\\'.$directory;
                }
            } else {
                switch($area) {
                    case 'core': return 'vendor/bin/phpunit --debug -c core '.$area.'/modules/'.$module.'/tests/src/'.$directory;
                    case 'contrib': return 'vendor/bin/phpunit --debug -c core modules/contrib/'.$module.'/tests/src/'.$directory;
                    case 'custom': return 'vendor/bin/phpunit --debug -c core modules/custom/'.$module.'/tests/src/'.$directory;
                }
            }
        }
    }

    /**
     * Builds the custom tests list.
     *
     * @param string $area
     *   The area the module is located. Core, contrib or custom.
     * @param string $path
     *   The absolute path the tests.
     *
     */
    private function _getCustomTests($area, $path) {
        $scan = scandir($path);
        unset($scan[array_search('.', $scan, true)]);
        unset($scan[array_search('..', $scan, true)]);
        foreach($scan as $module) {
            if (is_dir("$path/$module")) {
                $testsPath = "$path/$module/src/CustomTSTests/";
                if(is_dir($testsPath)) {
                    $this->_customTestsArray = glob($testsPath . '*.php');
                }
            }
        }
    }

    /**
     * Builds the menus directory list.
     *
     * @param string $area
     *   The area the module is located. Core, contrib or custom.
     * @param string $path
     *   The absolute path the tests.
     *
     */
    private function _getDirectories($area, $path) {
        $scan = scandir($path);
        unset($scan[array_search('.', $scan, true)]);
        unset($scan[array_search('..', $scan, true)]);
        foreach($scan as $module) {
            if (is_dir("$path/$module")) {
                $testsPath = "$path/$module/tests/src/";
                if(is_dir($testsPath)) {
                    $testsScan = scandir($testsPath);
                    unset($testsScan[array_search('.', $testsScan, true)]);
                    unset($testsScan[array_search('..', $testsScan, true)]);
                    foreach($testsScan as $directory) {
                        if (is_dir("$testsPath/$directory")) {
                            $this->_menuArray[$area][$module][] = $directory;
                        }
                    }
                }
            }
        }
    }

    /**
     * Builds the list of files in the phpunit directories.
     *
     * @param string $area
     *   The area the module is located. Core, contrib or custom.
     * @param string $moduleModules
     *   The module name.
     * @param string $moduleDirectory
     *   The test directory. Unit, Functional etc.
     * @param string $path
     *   The absolute path the tests.
     *
     */
    private function _getFilesInDirectories($area, $moduleModules, $moduleDirectory, $path) {
        $scan = scandir($path);
        unset($scan[array_search('.', $scan, true)]);
        unset($scan[array_search('..', $scan, true)]);
        foreach($scan as $module) {
            if($moduleModules == $module) {
                if (is_dir("$path/$module")) {
                    $testsPath = "$path/$module/tests/src";
                    if(is_dir($testsPath)) {
                        $testsScan = scandir($testsPath);
                        unset($testsScan[array_search('.', $testsScan, true)]);
                        unset($testsScan[array_search('..', $testsScan, true)]);
                        foreach($testsScan as $directory) {
                            if($moduleDirectory == $directory) {
                                if (is_dir("$testsPath/$directory")) {
                                    $globPath = "$path/$module/tests/src/$directory";
                                    $globedFiles = glob($globPath.'/*.php');
                                    $this->_listFilesArray[] = $this->getUrls($area, $moduleModules, $moduleDirectory);
                                    foreach($globedFiles as $file) {
                                        $this->_listFilesArray[] = $this->getUrls($area, $moduleModules, $moduleDirectory, $file);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Builds the reports lists.
     *
     * @param string $area
     *   The area the module is located. Core, contrib or custom.
     * @param string $moduleModules
     *   The module name.
     * @param string $moduleDirectory
     *   The test directory. Unit, Functional etc.
     * @param string $path
     *   The absolute path the tests.
     *
     */
    private function _getFormReportArray($area, $moduleModules, $moduleDirectory, $path) {
        $scan = scandir($path);
        unset($scan[array_search('.', $scan, true)]);
        unset($scan[array_search('..', $scan, true)]);
        foreach($scan as $module) {
            if (is_dir("$path/$module")) {
                $testsPath = "$path/$module/tests/src/";
                if(is_dir($testsPath)) {
                    $testsScan = scandir($testsPath);
                    unset($testsScan[array_search('.', $testsScan, true)]);
                    unset($testsScan[array_search('..', $testsScan, true)]);
                    foreach($testsScan as $directory) {
                        if($moduleDirectory == $directory) {
                            if (is_dir("$testsPath/$directory") && !isset($this->_formReportArray[$module])) {
                                if($moduleModules == 'all') {
                                    $this->_formReportArray[$module] = $this->getStatement($area, $module, $directory);
                                } else {
                                    if($moduleModules == $module) $this->_formReportArray[$module] = $this->getStatement($area, $module, $directory);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
